@@ -358,7 +358,7 @@ This structure is important since it is used in two different parts of the libra
     def __init__(self, *args, **kwargs):
         self.converter = kwargs.pop('converter',None) or keyconverter
         super(HashTable,self).__init__(*args, **kwargs)
-        
+    
     def __contains__(self, key):
         value = self.converter.tokey(key)
         if self._cache is None:
@@ -366,6 +366,10 @@ This structure is important since it is used in two different parts of the libra
         else:
             return value in self._cache
         
+    def __delitem__(self, key):
+        k = self.converter.tokey(key)
+        self.delete(k)
+        
     def add(self, key, value):
         '''Add ``key`` - ``value`` pair to hashtable.'''
         self.update({key:value})
@@ -456,6 +460,9 @@ This structure is important since it is used in two different parts of the libra
     
     def _mget(self, keys):
         raise NotImplementedError
+    
+    def delete(self, key):
+        raise NotImplementedError
 
     
 class TS(HashTable):
