@@ -1,14 +1,14 @@
 '''Two different implementation of a redis::map, a networked
 ordered associative container
 '''
-from itertools import izip, imap
-import base as structures
+from stdnet.utils import zip, iteritems
+from stdnet.backends.structures import base as structures
 
 
 def riteritems(self, com, *rargs):
     res = self.cursor.execute_command(com, self.id, *rargs)
     if res:
-        return izip(res[::2], res[1::2])
+        return zip(res[::2], res[1::2])
     else:
         return res
 
@@ -23,10 +23,24 @@ class List(structures.List):
         return self.cursor.execute_command('DEL', self.id)
     
     def pop_back(self):
-        return self.pickler.loads(self.cursor.execute_command('RPOP', self.id))
+        v = self.cursor.execute_command('RPOP', self.id)
+        if v:
+            return self.pickler.loads(v)
     
     def pop_front(self):
-        return self.pickler.loads(self.cursor.execute_command('LPOP', self.id))
+        v = self.cursor.execute_command('LPOP', self.id)
+        if v:
+            return self.pickler.loads(v)
+    
+    def block_pop_back(self, timeout = 0):
+        v = self.cursor.execute_command('BRPOP', self.id, timeout)
+        if v:
+            return self.pickler.loads(v[1])
+    
+    def block_pop_front(self, timeout = 0):
+        v = self.cursor.execute_command('BLPOP', self.id, timeout)
+        if v:
+            return self.pickler.loads(v[1])
     
     def _all(self):
         return self.cursor.execute_command('LRANGE', self.id, 0, -1)
@@ -55,7 +69,7 @@ class Set(structures.Set):
     def clear(self):
         return self.delete()
     
-    def discard(self, elem):
+    def _discard(self, elem):
         return self.cursor.execute_command('SREM', self.id, elem)
     
     def _save(self):
@@ -81,7 +95,7 @@ class OrderedSet(structures.OrderedSet):
         '''Size of set'''
         return self.cursor.execute_command('ZCARD', self.id)
     
-    def discard(self, elem):
+    def _discard(self, elem):
         return self.cursor.execute_command('ZREM', self.id, elem)
     
     def _contains(self, value):
@@ -128,7 +142,7 @@ class HashTable(structures.HashTable):
         return self.cursor.execute_command('HKEYS', self.id)
     
     def _items(self):
-        return self.cursor.execute_command('HGETALL', self.id).iteritems()
+        return iteritems(self.cursor.execute_command('HGETALL', self.id))
         #return riteritems(self, 'HGETALL', self.id)
 
     def values(self):
@@ -137,60 +151,57 @@ class HashTable(structures.HashTable):
             
     def _save(self):
         items = []
-        [items.extend(item) for item in self.pipeline.iteritems()]
+        [items.extend(item) for item in iteritems(self.pipeline)]
         return self.cursor.execute_command('HMSET',self.id,*items)
     
     def add_expiry(self):
         self.cursor.execute_command('EXPIRE', self.id, self.timeout)
         
 
-class Map(structures.Map):
+class TS(structures.TS):
     '''Requires Redis Map structure which is not yet implemented in redis (and I don't know if it will).
 It is implemented on my redis-fork at https://github.com/lsbardel/redis'''
     def _size(self):
-        return self.cursor.execute_command('TLEN', self.id)
+        return self.cursor.execute_command('TSLEN', self.id)
     
     def clear(self):
         return self.cursor.execute_command('DEL', self.id)
     
     def _get(self, key):
-        return self.cursor.execute_command('TGET', self.id, key)
+        return self.cursor.execute_command('TSGET', self.id, key)
     
     def _mget(self, keys):
         return self.cursor.execute_command('TMGET', self.id, *keys)
     
     def delete(self, key):
-        return self.cursor.execute_command('TDEL', self.id, key)
+        return self.cursor.execute_command('TSDEL', self.id, key)
     
     def _contains(self, key):
-        if self.cursor.execute_command('TEXISTS', self.id, key):
-            return True
-        else:
-            return False
+        return self.cursor.execute_command('TSEXISTS', self.id, key)
         
     def _getdate(self, val):
         return None if not val else val[0]
         
     def _irange(self, start, end):
-        return riteritems(self, 'TRANGE', start, end, 'withscores')
+        return riteritems(self, 'TSRANGE', start, end, 'withtimes')
     
     def _range(self, start, end):
-        return riteritems(self, 'TRANGEBYSCORE', start, end, 'withscores')
+        return riteritems(self, 'TSRANGEBYTIME', start, end, 'withtimes')
     
     def _count(self, start, end):
-        return self.cursor.execute_command('TCOUNT', self.id, start, end)
+        return self.cursor.execute_command('TSCOUNT', self.id, start, end)
     
     def _front(self):
-        return self._getdate(self.cursor.execute_command('TRANGE', self.id, 0, 0, 'novalues'))
+        return self._getdate(self.cursor.execute_command('TSRANGE', self.id, 0, 0, 'novalues'))
     
     def _back(self):
-        return self._getdate(self.cursor.execute_command('TRANGE', self.id, -1, -1, 'novalues'))
+        return self._getdate(self.cursor.execute_command('TSRANGE', self.id, -1, -1, 'novalues'))
         
     def _keys(self):
-        return self.cursor.execute_command('TRANGE', self.id, 0, -1, 'novalues')
+        return self.cursor.execute_command('TSRANGE', self.id, 0, -1, 'novalues')
     
     def _items(self):
-        return riteritems(self, 'TRANGE', 0, -1, 'withscores')
+        return riteritems(self, 'TSRANGE', 0, -1, 'withtimes')
 
     def values(self):
         for ky,val in self.items():
@@ -198,8 +209,8 @@ It is implemented on my redis-fork at https://github.com/lsbardel/redis'''
             
     def _save(self):
         items = []
-        [items.extend(item) for item in self.pipeline.iteritems()]
-        return self.cursor.execute_command('TADD',self.id,*items)
+        [items.extend(item) for item in iteritems(self.pipeline)]
+        return self.cursor.execute_command('TSADD',self.id,*items)
     
     def add_expiry(self):
         self.cursor.execute_command('EXPIRE', self.id, self.timeout)
