@@ -1,11 +1,14 @@
 import sys
 import copy
-from itertools import izip
-from fields import Field, AutoField
 
+from stdnet.utils import zip, to_bytestring
 from stdnet.orm import signals
 from stdnet.exceptions import *
-from query import UnregisteredManager 
+
+from .globals import hashmodel
+from .query import UnregisteredManager
+from .fields import Field, AutoField
+
 
 def get_fields(bases, attrs):
     fields = {}
@@ -13,18 +16,17 @@ def get_fields(bases, attrs):
         if hasattr(base, '_meta'):
             fields.update(copy.deepcopy(base._meta.dfields))
     
-    for name,field in attrs.items():
+    for name,field in list(attrs.items()):
         if isinstance(field,Field):
             fields[name] = attrs.pop(name)
     
     return fields
 
 
-
 class Metaclass(object):
     '''Utility class used for storing all information
-which maps a :class:`stdnet.orm.StdModel` model
-into a :class:`stdnet.HashTable` structure in a :class:`stdnet.BackendDataServer`.
+which maps a :class:`stdnet.orm.StdModel` model into an object in the
+in the remote :class:`stdnet.BackendDataServer`.
 An instance is initiated when :class:`stdnet.orm.StdModel` class is created:
 
 .. attribute:: model
@@ -51,34 +53,38 @@ An instance is initiated when :class:`stdnet.orm.StdModel` class is created:
     primary key ::class:`stdnet.orm.Field`
 
 '''
+    VALATTR = '_validation'
     def __init__(self, model, fields,
                  abstract = False, keyprefix = None,
                  app_label = '', verbose_name = None, **kwargs):
-        self.abstract  = abstract
+        self.abstract = abstract
         self.keyprefix = keyprefix
-        self.model     = model
+        self.model = model
         self.app_label = app_label
-        self.name      = model.__name__.lower()
-        self.fields       = []
+        self.name = model.__name__.lower()
+        self.fields = []
         self.scalarfields = []
-        self.multifields  = []
-        self.dfields      = {}
-        self.timeout      = 0
-        self.related      = {}
+        self.multifields = []
+        self.dfields = {}
+        self.timeout = 0
+        self.related = {}
         self.verbose_name = verbose_name or self.name
-        self.maker        = lambda : model.__new__(model)
-        model._meta       = self
+        self.maker = lambda : model.__new__(model)
+        model._meta = self
+        hashmodel(model)
         
+        # Check if ID field exists
         try:
             pk = fields['id']
         except:
+            # ID field not available, create one
             pk = AutoField(primary_key = True)
         pk.register_with_model('id',model)
         self.pk = pk
         if not self.pk.primary_key:
             raise FieldError("Primary key must be named id")
         
-        for name,field in fields.iteritems():
+        for name,field in fields.items():
             if name == 'id':
                 continue
             field.register_with_model(name,model)
@@ -98,26 +104,49 @@ An instance is initiated when :class:`stdnet.orm.StdModel` class is created:
         return self.__repr__()
         
     def basekey(self, *args):
-        '''Calculate the key to access model hash-table, and model filters in the database.
-        For example::
+        """Calculate the key to access model hash-table/s,
+and model filters in the database.
+The key is an encoded binary string. For example::
         
-            >>> a = Author(name = 'Dante Alighieri').save()
-            >>> a.meta.basekey()
-            'stdnet:author'
-            '''
-        key = '%s%s' % (self.keyprefix,self.name)
+    >>> from examples.models import User
+    >>> from orm import register
+    >>> register(User)
+    'redis db 7 on 127.0.0.1:6379'
+    >>> User._meta.basekey()
+    b'stdnet.examples.user'
+    >>> a = Author(name = 'Dante Alighieri').save()
+    >>> a.meta.basekey()
+    b'stdnet.someappname.author'
+    """
+        key = '%s%s' % (self.keyprefix,self)
         for arg in args:
             key = '%s:%s' % (key,arg)
-        return key
+        return to_bytestring(key)
     
     def autoid(self):
+        '''The id for autoincrements ids'''
         return self.basekey('ids')
     
-    @property
-    def uniqueid(self):
-        '''Unique id for an instance. This is unique across multiple model types.'''
-        return self.basekey(self.id)
-    
+    def is_valid(self, instance):
+        '''Perform validation and stored serialized data, indexes and errors.
+Return ``True`` is the instance is ready to be saved to database.'''
+        v = {}
+        setattr(instance,self.VALATTR,v)
+        data = v['data'] = {}
+        indexes = v['indices'] = []
+        errors = v['errors'] = {}
+        #Loop over scalar fields first
+        for field in self.scalarfields:
+            name = field.attname
+            svalue = field.serialize(getattr(instance,name,None))
+            if (svalue is None or svalue is '') and field.required:
+                errors[name] = "Field '{0}' is required for '{1}'.".format(name,self)
+            else:
+                data[name] = svalue
+                if field.index:
+                    indexes.append((field,svalue))
+        return len(errors) == 0
+                
     def table(self):
         '''Return an instance of :class:`stdnet.HashTable` holding
 the model table'''
@@ -125,15 +154,18 @@ the model table'''
             raise ModelNotRegistered('%s not registered. Call orm.register(model_class) to solve the problem.' % self)
         return self.cursor.hash(self.basekey(),self.timeout)
     
-    def make(self, id, data):
-        '''Create a model instance from server data'''
-        obj = self.maker()
-        setattr(obj,'id',id)
-        if data:
-            for field,value in izip(self.scalarfields,data):
-                setattr(obj,field.attname,field.to_python(value))
-        obj.afterload()
-        return obj
+    def flush(self, count = None):
+        '''Fast method for clearing the whole table including related tables'''
+        for rel in self.related.values():
+            rmeta = rel.to._meta
+            # This avoid circular reference
+            if rmeta is not self:
+                rmeta.flush(count)
+        if self.cursor:
+            self.cursor.flush(self, count)
+
+    def database(self):
+        return self.cursor
 
 
 class StdNetType(type):
@@ -141,22 +173,28 @@ class StdNetType(type):
     def __new__(cls, name, bases, attrs):
         super_new = super(StdNetType, cls).__new__
         parents = [b for b in bases if isinstance(b, StdNetType)]
-        if not parents:
+        if not parents or attrs.pop('is_base_class',False):
             return super_new(cls, name, bases, attrs)
         
         # remove the Meta class if present
         meta      = attrs.pop('Meta', None)
+        if meta:
+            kwargs   = meta_options(**meta.__dict__)
+        else:
+            kwargs   = meta_options()
+        
+        #if kwargs['abstract']:
+        #    return super_new(cls, name, bases, attrs)
+        
         # remove and build field list
         fields    = get_fields(bases, attrs)        
         # create the new class
         objects   = attrs.pop('objects',None)
         new_class = super_new(cls, name, bases, attrs)
         new_class.objects = objects
-        if meta:
-            kwargs   = meta_options(**meta.__dict__)
-        else:
-            kwargs   = {}
-        if kwargs.pop('app_label',None) is None:
+        app_label = kwargs.pop('app_label')
+        
+        if app_label is None:
             model_module = sys.modules[new_class.__module__]
             try:
                 app_label = model_module.__name__.split('.')[-2]
@@ -170,10 +208,12 @@ class StdNetType(type):
         return new_class
     
 
-
 def meta_options(abstract = False,
                  keyprefix = None,
+                 app_label = None,
                  **kwargs):
     return {'abstract': abstract,
-            'keyprefix': keyprefix}
-    
\ No newline at end of file
+            'keyprefix': keyprefix,
+            'app_label':app_label}
+    
+
